/*
  Macro           Test_SetFont_LowLevel
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   Windows GUI TSE 4.0 upwards
  Version         v1   8 Feb 2025


  This macro aims to help to resolve a SetFont() bug.

  Dd 8 Feb 2025 TSE's SetFont() always returns 1.
  It should return 0 for a non-existing font name.

  The only low-level way I found, is to test if the new textface (font name)
  equals the requested font name.

*/





// Start of compatibility restrictions and mitigations


#ifdef LINUX
  This extension is not LINUX compatible.
#endif

#ifdef WIN32
#else
   16-bit versions of TSE are not supported. You need at least GUI TSE v4.
#endif

#ifdef EDITOR_VERSION
#else
   Editor Version is older than TSE 3.0. You need at least GUI TSE v4.
#endif

#if EDITOR_VERSION < 4000h
   Editor Version is older than TSE 4.0. You need at least GUI TSE v4.
#endif


// End of compatibility restrictions and mitigations.





// Constants for CreateFont() from wingdi.h:

#define ANSI_CHARSET                0
#define DEFAULT_CHARSET             1

#define OUT_DEFAULT_PRECIS          0
#define OUT_STRING_PRECIS           1
#define OUT_CHARACTER_PRECIS        2
#define OUT_STROKE_PRECIS           3
#define OUT_TT_PRECIS               4
#define OUT_DEVICE_PRECIS           5
#define OUT_RASTER_PRECIS           6
#define OUT_TT_ONLY_PRECIS          7
#define OUT_OUTLINE_PRECIS          8
#define OUT_SCREEN_OUTLINE_PRECIS   9
#define OUT_PS_ONLY_PRECIS         10

#define CLIP_DEFAULT_PRECIS         0

#define DEFAULT_QUALITY             0
#define DRAFT_QUALITY               1
#define PROOF_QUALITY               2
#define NONANTIALIASED_QUALITY      3
#define ANTIALIASED_QUALITY         4
#define CLEARTYPE_QUALITY           5
#define CLEARTYPE_NATURAL_QUALITY   6

#define DEFAULT_PITCH               0
#define FIXED_PITCH                 1
#define VARIABLE_PITCH              2

#define FF_DONTCARE   (0 shl 4)  /* Don't care or don't know. */
#define FF_ROMAN      (1 shl 4)  /* Variable stroke width, serifed. */
                                 /* Times Roman, Century Schoolbook, etc. */
#define FF_SWISS      (2 shl 4)  /* Variable stroke width, sans-serifed. */
                                 /* Helvetica, Swiss, etc. */
#define FF_MODERN     (3 shl 4)  /* Constant stroke width, serifed or sans-serifed. */
                                 /* Pica, Elite, Courier, etc. */
#define FF_SCRIPT     (4 shl 4)  /* Cursive, etc. */
#define FF_DECORATIVE (5 shl 4)  /* Old English, etc. */





// Windows APIs

dll "<user32.dll>"
  integer proc GetDC(integer hWnd)
  integer proc ReleaseDC(integer hWnd, integer hDC)
end


dll "<gdi32.dll>"

  integer proc CreateFontA(
    integer cHeight,
    integer cWidth,
    integer cEscapement,
    integer cOrientation,
    integer cWeight,
    integer bItalic,
    integer bUnderline,
    integer bStrikeOut,
    integer iCharSet,
    integer iOutPrecision,
    integer iClipPrecision,
    integer iQuality,
    integer iPitchAndFamily,
    string  pszFaceName:cstrval)

  integer proc DeleteObject(
    integer h)

  integer proc GetTextFace(
    integer hdc,
    integer c,
    var string lpName:strptr) : "GetTextFaceA"   // LPSTR

  integer proc SelectObject(
    integer hdc,
    integer h)

end



integer proc set_font(string font_name, integer font_size, integer font_flags)
  integer device_context_handle        = FALSE
  integer dummy                        = 0
  integer new_font_handle              = FALSE
  integer old_font_flags               = 0
  integer old_font_height              = 0
  string  old_font_name [MAXSTRINGLEN] = ''
  integer old_font_object              = 0
  integer old_font_size                = 0
  integer old_font_width               = 0
  integer result                       = FALSE
  string  textface      [MAXSTRINGLEN] = Format('':255:' ')
  integer textface_bytes               = 0

  if TRUE
    device_context_handle = GetDC(GetWinHandle())
  else
    device_context_handle = GetDC(0)
  endif

  GetFont(old_font_name, old_font_size, old_font_flags)

  // For this test skip font_size and font_flags
  dummy = font_size + font_flags + dummy
  GetCharWidthHeight(old_font_width, old_font_height)

  new_font_handle = CreateFontA(old_font_height,
                                old_font_width,
                                0, 0, 0, 0, 0, 0,
                                // The below field's 0 constants:
                                //    ANSI_CHARSET,
                                //    OUT_DEFAULT_PRECIS,
                                //    CLIP_DEFAULT_PRECIS,
                                //    DEFAULT_QUALITY,
                                //    DEFAULT_PITCH | FF_DONTCARE,
                                // Their actually used values:
                                      DEFAULT_CHARSET,
                                      OUT_DEFAULT_PRECIS,
                                      CLIP_DEFAULT_PRECIS,
                                      DEFAULT_QUALITY,
                                      DEFAULT_PITCH | FF_DONTCARE,
                                font_name)

  old_font_object = SelectObject(device_context_handle, new_font_handle)
  textface_bytes  = GetTextFace(device_context_handle, MAXSTRINGLEN, textface)
  textface = textface[1: textface_bytes]

  // Cleanup:
  SelectObject(device_context_handle, old_font_object)
  DeleteObject(new_font_handle)
  ReleaseDC(GetWinHandle(), device_context_handle)

  result = EquiStr(textface, font_name)
  return(result)
end set_font


proc test_set_font(string font_name)
  AddLine(Format(set_font(font_name, 12, 0); font_name))
end test_set_font


proc Main()
  if isGUI()
    NewFile()
    test_set_font('Arial')
    test_set_font('Consolas')
    test_set_font('Courier New')
    test_set_font('Fixedsys')
    test_set_font('Terminal')
    test_set_font('MyAwesomeVont')
    test_set_font('Lucida Console')
    test_set_font('Your Awesome Vont')
    test_set_font('Lucida Sans Typewriter')
    test_set_font('Not A Vont')
    BegFile()
  else
    Warn('Not compatible with Console TSE.')
  endif

  PurgeMacro(CurrMacroFilename())
end Main

